#include "main10V.h"
#include "../ui_main10V.h"

Main10VWidget::Main10VWidget(QWidget *parent)
    : QWidget(parent), ui(new Ui::Main10VWidgetClass)
{
    boardType = 0;

    ui->setupUi(this);
    connect(ui->ButtonLoad, SIGNAL(clicked()), this, SLOT(slot_readParameters()));
    connect(ui->ButtonUpdate, SIGNAL(clicked()), this, SLOT(slot_sendParameters()));

    ui->ButtonUpdate->setEnabled(false);
    ui->ButtonUpdate->setToolTip("Start by loading the board's parameters before updating them");
    timer = new QTimer;

    redPal = new QPalette(ui->p->palette());
    greenPal = new QPalette(ui->p->palette());

    redPal->setColor(QPalette::Base, QColor("orange"));
    greenPal->setColor(QPalette::Base, QColor("lightgreen"));

    paramsUpdated = false;
    currentParam = -1;

    connect(ui->p, SIGNAL(valueChanged(double)), this, SLOT(slot_changedP()));
    connect(ui->i, SIGNAL(valueChanged(double)), this, SLOT(slot_changedI()));
    connect(ui->d, SIGNAL(valueChanged(double)), this, SLOT(slot_changedD()));
    connect(ui->commandMultiplier, SIGNAL(valueChanged(QString)), this, SLOT(slot_changedCommandMultiplier()));
    connect(ui->faultError, SIGNAL(valueChanged(QString)), this, SLOT(slot_changedFaultError()));
    connect(ui->deadBand, SIGNAL(valueChanged(QString)), this, SLOT(slot_changedDeadBand()));
    connect(ui->paramFF0, SIGNAL(valueChanged(double)), this, SLOT(slot_changedFF0()));
    connect(ui->paramFF1, SIGNAL(valueChanged(double)), this, SLOT(slot_changedFF1()));
    connect(ui->paramTPSC, SIGNAL(valueChanged(QString)), this, SLOT(slot_changedTCPS()));
    connect(ui->maxOutput, SIGNAL(valueChanged(double)), this, SLOT(slot_changedMaxOutput()));


}

Main10VWidget::~Main10VWidget()
{
    delete ui;
}


 void Main10VWidget::slot_readParameters()
 {
    ui->ButtonUpdate->setToolTip(tr("Write the new parameters into the Servo Board's EEProm"));
    ui->ButtonUpdate->setEnabled(true);

    emit sig_SendDataToBoard(QString("l\r\n"));
 }

 void Main10VWidget::slot_sendParameters()
 {
    static bool doNotRemind = false;
    currentParam = 0;
    if(doNotRemind == false)
    {
	QMessageBox warning(QMessageBox::Warning, QString(tr("Warning")), QString(tr("Please note that changin the parameters can be dangerous, be sure to understand what you are doing. <br>Proceed?")),
			    QMessageBox::YesToAll|QMessageBox::Yes|QMessageBox::Abort, this, 0);
	int ret = warning.exec();
	if(ret == QMessageBox::YesToAll)
	    doNotRemind = true;

	if(ret == QMessageBox::Abort)
	    return;
    }
    sendNextParam();
//    emit sig_SendDataToBoard(QString("Current Settings:\r\nservo enabled = 1\r\n(p) = 0.020000\r\n(i) = 0.000000\r\n(d) = 0.000050\r\nFF(0) = 0.000000\r\nFF(1) = 0.000000\r\ndead(b)and = 1.000000\r\n(m)ax output = 2.500000\r\n(f)ault error = 1024.000000\r\n(x)pc cmd multiplier = 3\r\n(t)icks per servo cycle= 10\r\n"));
 }

 void Main10VWidget::sendNextParam()
 {
     switch (currentParam){
	 case 0:
	    emit sig_SendDataToBoard(QString('p').append(QString::number(ui->p->value())).append("\r\n"));
	    break;
	 case 1:
	    emit sig_SendDataToBoard(QString('i').append(QString::number(ui->i->value())).append("\r\n"));
	    break;
	 case 2:
	    emit sig_SendDataToBoard(QString('d').append(QString::number(ui->d->value())).append("\r\n"));
	    break;
	 case 3:
	    emit sig_SendDataToBoard(QString('0').append(QString::number(ui->paramFF0->value())).append("\r\n"));
	    break;
	 case 4:
	    emit sig_SendDataToBoard(QString('1').append(QString::number(ui->paramFF1->value())).append("\r\n"));
	    break;
	 case 5:
            if(boardType == 0)
                emit sig_SendDataToBoard(QString('m').append(QString::number(ui->maxOutput->value()*10)).append("\r\n"));
            if(boardType == 1)
                emit sig_SendDataToBoard(QString('m').append(QString::number(ui->maxOutput->value())).append("\r\n"));

	    break;
	 case 6:
	    emit sig_SendDataToBoard(QString('t').append(QString::number(ui->paramTPSC->value())).append("\r\n"));
	    break;
	 case 7:
	    emit sig_SendDataToBoard(QString('x').append(QString::number(ui->commandMultiplier->value())).append("\r\n"));
	    break;
	 case 8:
	    emit sig_SendDataToBoard(QString('b').append(QString::number(ui->deadBand->value())).append("\r\n"));
	    break;
	 case 9:
	    emit sig_SendDataToBoard(QString('f').append(QString::number(ui->faultError->value())).append("\r\n"));
	    break;
	 default :
	    currentParam--;
	    break;
     }
     currentParam++;
 }

 void Main10VWidget::slot_NewData(QString data)
 {
     bool paramUpdated = false;

    if(data.indexOf("(p)") != -1)
    {
        ui->p->setValue(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("(p)"))).toDouble());
	ui->p->setPalette(*greenPal);
	paramUpdated = true;
    }
    if(data.indexOf("(i)") != -1)
    {
        ui->i->setValue(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("(i)"))).toDouble());
	ui->i->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(d)") != -1)
    {
        ui->d->setValue(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("(d)"))).toDouble());
	ui->d->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(0)") != -1)
    {
        ui->paramFF0->setValue(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("(0)") + 5)).toDouble());
	ui->paramFF0->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(1)") != -1)
    {
        ui->paramFF1->setValue(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("(1)") + 5)).toDouble());
	ui->paramFF1->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(b)") != -1)
    {
        ui->deadBand->setValue(data.mid(data.indexOf(QRegExp("(\\d+)"), data.indexOf("(b)"))).toDouble());
	ui->deadBand->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(m)") != -1)
    {
        data[data.indexOf('%')] = ' ';
        if(boardType == 0)
            ui->maxOutput->setValue(data.mid(data.indexOf(QRegExp("(\\d+)"), data.indexOf("(m)"))).toDouble());
        if(boardType == 1)
            ui->maxOutput->setValue(data.mid(data.indexOf(QRegExp("(\\d+)"), data.indexOf("(m)"))).toDouble());
	ui->maxOutput->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(f)") != -1)
    {
        ui->faultError->setValue(data.mid(data.indexOf(QRegExp("(\\d+)"), data.indexOf("(f)"))).toDouble());
	ui->faultError->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(x)") != -1)
    {
        ui->commandMultiplier->setValue(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("(x)"))).toDouble());
	ui->commandMultiplier->setPalette(*greenPal);
	paramUpdated = true;
    }

    if(data.indexOf("(t)") != -1)
    {
        ui->paramTPSC->setValue(data.mid(data.indexOf(QRegExp("(\\d+)"), data.indexOf("(t)"))).toDouble());
	ui->paramTPSC->setPalette(*greenPal);
	paramUpdated = true;
    }
    if(paramUpdated == true)
    {
	sendNextParam();	
	ui->ButtonUpdate->setEnabled(true);
    }
 }

 void Main10VWidget::Main10VWidget::slot_changedP()
 {
     ui->p->setPalette(*redPal);
 }

void Main10VWidget::slot_changedI()
 {
     ui->i->setPalette(*redPal);
 }

void Main10VWidget::slot_changedD()
 {
     ui->d->setPalette(*redPal);
 }

void Main10VWidget::slot_changedDeadBand()
 {
    ui->deadBand->setPalette(*redPal);
 }

void Main10VWidget::slot_changedFF0()
 {
     ui->paramFF0->setPalette(*redPal);
 }

void Main10VWidget::slot_changedFF1()
 {
     ui->paramFF1->setPalette(*redPal);
 }

void Main10VWidget::slot_changedTCPS()
 {
     ui->paramTPSC->setPalette(*redPal);
 }

void Main10VWidget::slot_changedMaxOutput()
 {
     ui->maxOutput->setPalette(*redPal);
 }

void Main10VWidget::slot_changedFaultError()
 {
     ui->faultError->setPalette(*redPal);
 }

void Main10VWidget::slot_changedCommandMultiplier()
 {
     ui->commandMultiplier->setPalette(*redPal);
 }
