/********************************************************************************************/
/*  class YTTLINK                                                                           */
/*                                                                                          */
/*  Encapsulates a link between a board and YTT                                             */
/*                                                                                          */
/*                                                                                          */
/*                                                                                          */
/*                                                                                          */
/********************************************************************************************/
#ifdef WIN32
    #include <qextserialenumerator.h>
#else
    #include <QDir>
    #include <QFileInfoList>
#endif
#include <QDockWidget>
#include "yttlink.h"
#include "ui_mainwindow.h"
#include "ui_logview.h"

#include "../ui_serconn.h"

extern bool NO_LOG;

YTTLINK::YTTLINK(QObject *par)
{
    setParent(par);
    status = closed;
    board = new BOARD(0);
    board->MainWidget->setEnabled(false);
    board->MonitorWidget->setEnabled(false);

    LinkMenu = new LinkSerialWidget;
    logViewer = new YTTLinkLogView();
    //logViewer->hide();

    logToolBox=new QDockWidget(tr("Log View"),(QWidget*)this->parent());
    logToolBox->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea | Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea);
    logToolBox->setWidget(logViewer);
    QMainWindow *mainW = (QMainWindow*) this->parent();
    mainW->addDockWidget(Qt::LeftDockWidgetArea, logToolBox);
    //logToolBox->setGeometry(0,0,400,200);
    logToolBox->hide();
    //if(this->parent())
      //  logToolBox->setFloating(false);

    port = new QextSerialPort();

    timer = new QTimer;
    timer->setSingleShot (false);
    timer->setInterval(100);

    timeoutTimer = new QTimer;
    timeoutTimer->setSingleShot (true);
    timeoutTimer->setInterval(5000);	// connection timout 5s

    receivedData = QString("");
    receivedData.reserve(10000);


    connect(this, SIGNAL(sig_newLineReceived(QString)), board, SLOT(slot_NewDataFromBoard(QString)));
    connect(this, SIGNAL(sig_newLineReceived(QString)), logViewer, SLOT(slot_WriteRawText(QString)));
    connect(this, SIGNAL(sig_StatusChanged(int, QString)), logViewer, SLOT(slot_AddLine(int,QString)));
    connect(LinkMenu->ui->ButtonConnect, SIGNAL(clicked()), this, SLOT(Open()));
    connect(timer, SIGNAL(timeout()), this, SLOT(verifyNewData()));
    connect(logViewer, SIGNAL(sig_DataToSend(QString)), this, SLOT(slot_SendLine(QString)));
    connect(board->MainWidget, SIGNAL(sig_SendLineToBoard(QString)), this, SLOT(slot_SendLine(QString)));
    connect(board->MainWidget, SIGNAL(sig_SendDataToBoard(QString)), this, SLOT(slot_Send(QString)));
    connect(board->MonitorWidget, SIGNAL(sig_SendData(QString)), this, SLOT(slot_Send(QString)));
    connect(timeoutTimer, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
    connect(LinkMenu->ui->BoardModel, SIGNAL(currentIndexChanged(int)), this, SLOT(slot_boardModelChanged(int)));
}

YTTLINK::~YTTLINK()
{
    this->Close();			// close the link
    delete board;
    delete LinkMenu;
    delete logViewer;
}

void YTTLINK::slot_LogConsole()
{
    if(logToolBox)
        logToolBox->show();
}

int YTTLINK::Open()
{

    if(status == closed)
    {
	port->close();
	QString portN = LinkMenu->ui->COMPortNumber->currentText();
#ifdef WIN32
	portN = portN.remove(0, portN.indexOf('(')+1);
	portN = portN.left(portN.indexOf(')'));

	port->setPortName(QString("\\\\.\\").append(portN));
#else
	portN=portN.remove(0, portN.indexOf("/dev"));
	port->setPortName(portN);
#endif
	port->setBaudRate(BAUD57600);
	port->setFlowControl(FLOW_OFF);
	port->setParity(PAR_NONE);
	port->setDataBits(DATA_8);
	port->setStopBits(STOP_1);
	//set timeouts to 500 ms
#ifdef WIN32
        port->setTimeout(1);
#endif
	port->open(QIODevice::ReadWrite);
	port->setBaudRate(BAUD57600);
        port->setFlowControl(FLOW_OFF);
        port->setParity(PAR_NONE);
        port->setDataBits(DATA_8);
        port->setStopBits(STOP_1);

	if(port->isOpen()==false)
	{
	    int err = port->lastError();
//	    QString errS = errors.at(err);
	    emit sig_StatusChanged(iConnect, QString(tr("Error while opening ")).append(LinkMenu->ui->COMPortNumber->currentText()).append(tr(". Port not opened!")));
	    return -1;
	}
	port->setRts(true); // release the device from reset
	status = opened;
	board->MainWidget->setEnabled(true);
	board->MonitorWidget->setEnabled(true);


	emit sig_StatusChanged(iConnect, QString(tr("Connection opened: ")).append(LinkMenu->ui->COMPortNumber->currentText()));
	LinkMenu->ui->ButtonConnect->setText(QString(tr("Disconnect")));
	LinkMenu->ui->ButtonConnect->setIcon(QIcon(":/images/disconnect.png"));
        LinkMenu->ui->COMPortNumber->setEnabled(false);
        LinkMenu->ui->BoardModel->setEnabled(false);

	timer->start();
	return 0;
    }
    else
    {
	char buff[1024];
	int numBytes;

	timer->stop();
	port->flush();
	port->close();

	emit sig_StatusChanged(iDisconnect, QString(tr("Connection closed")));
	LinkMenu->ui->ButtonConnect->setText(QString(tr("Connect")));
	LinkMenu->ui->ButtonConnect->setIcon(QIcon(":/images/connect.png"));
	LinkMenu->ui->COMPortNumber->setEnabled(true);
        LinkMenu->ui->BoardModel->setEnabled(true);
	status = closed;
	board->MainWidget->setEnabled(false);
	board->MonitorWidget->setEnabled(false);

    }
    return 0;
}

int YTTLINK::Close(void)
{
    return 0;
}

bool YTTLINK::IsOpen(void)
{
    if(status == closed)
	return false;
    return true;
}

int YTTLINK::slot_Send(QString text)
{
    int i = port->write(text.toAscii(), text.length());
    port->flush();
    emit sig_StatusChanged(iSend, QString::number(text.toAscii().length(), 10).append(tr(" bytes sent")));
//    emit sig_Receive(text);
    timeoutTimer->start(3000);
    return i;
}

int YTTLINK::slot_SendLine(QString text)
{
    int i = port->write(text.append("\r\n").toAscii(), text.length());
    port->flush();
    emit sig_StatusChanged(iSend, QString::number(text.toAscii().length(), 10).append(tr("-bytes line sent")));
//    emit sig_Receive(text);
    timeoutTimer->start(3000);
    return i;
}

void YTTLINK::slot_boardModelChanged(int a)
{
    if(LinkMenu->ui->BoardModel->currentText() == "YAPSC:10V")
    {
        board->MainWidget->ui->maxOutput->setMaximum(10.0);
        board->MainWidget->ui->maxOutput->setSuffix(" Volts");
        board->MainWidget->boardType = 0;
    }
    if(LinkMenu->ui->BoardModel->currentText() == "YAPSC:V1")
    {
        board->MainWidget->ui->maxOutput->setMaximum(100.0);
        board->MainWidget->ui->maxOutput->setSuffix(" %");
        board->MainWidget->boardType = 1;
    }
}


QWidget* YTTLINK::Menu()
{
#ifdef WIN32
    QList<QextPortInfo> ports = QextSerialEnumerator::getPorts();
    for (int i = 0; i < ports.size(); i++)
    {
	LinkMenu->AddPortNum(ports.at(i).friendName, ports.at(i).portName);
    }
#else // _UNIX
    QDir dir("/dev");
    QStringList filters;

    filters << "ttyS*";
    dir.setNameFilters(filters);
    dir.setFilter(QDir::Files | QDir::System);
    QFileInfoList list = dir.entryInfoList();

    for (int i=0; i< list.size(); i++)
    {
	LinkMenu->AddPortNum(list.at(i).canonicalFilePath (), list.at(i).canonicalFilePath ());
    }


    QStringList filtersUSB;
    filtersUSB << "ttyUSB*";
    dir.setNameFilters(filtersUSB);
    dir.setFilter(QDir::Files | QDir::System);
    QFileInfoList listUSB = dir.entryInfoList();

    for (int i=0; i< listUSB.size(); i++)
    {
	LinkMenu->AddPortNum(QString(tr("UBS serial port ")).append(listUSB.at(i).canonicalFilePath ()), "");
    }
#endif

    return LinkMenu;
}

int YTTLINK::BoardNumber()
{
    return 1;
}

QWidget* YTTLINK::ConfigWidget(int boardNum, int subPart)
{
    if(boardNum >0)
	return 0;
    return board->subPart(subPart);
}

void YTTLINK::verifyNewData()
{
    if(!port->isOpen())
	return;
    if(port->bytesAvailable() < 1)
	return;

    timeoutTimer->stop();

    char buff[1024];
    int numBytes;
    numBytes = port->bytesAvailable();

    if(numBytes > 1024)
    	numBytes = 1024;

    int i = port->read(buff, numBytes);
    if (i != -1)
		buff[i] = '\0';
	else
		buff[0] = '\0';
    QString msg = buff;

    if(receivedData.size() + i > 10000)
	return;
    receivedData.append(msg);
//    logViewer->slot_WriteRawText(msg);
    if(receivedData.indexOf('\n') < 1)
	return;

//    logViewer->slot_AddLine(0, QString(buffer + currIndex));

//    QString newLine(buffer+j);
//    emit sig_newLineReceived(newLine);

    QStringList lines = receivedData.split('\n', QString::KeepEmptyParts);
    for(int i = 0; i < lines.count()-1; i++)
    {
	emit sig_newLineReceived(lines.at(i));
	logViewer->slot_AddLine(iReceive, QString(tr("Received data : ")).append(lines.at(i)));
    }
    receivedData = lines.last();
}

void YTTLINK::connectionTimeout()
{
    emit sig_StatusChanged(iError, QString(tr("Connection timeout!")));
}

/********************************************************************************************/
/*                                                                                          */
/*                                      For GUI                                             */
/*                                                                                          */
/********************************************************************************************/

LinkSerialWidget::LinkSerialWidget(QWidget *parent)
    : QWidget(parent), ui(new Ui::LinkSerialWidgetClass)
{
    ui->setupUi(this);
}

LinkSerialWidget::~LinkSerialWidget()
{
    delete ui;
}

void LinkSerialWidget::AddPortNum(QString portFriendlyName, QString portName)
{
    ui->COMPortNumber->addItem(portFriendlyName);
    ui->COMPortNumber->setCurrentIndex(ui->COMPortNumber->count()-1);
}

int LinkSerialWidget::Type()
{
    return ui->BoardModel->currentIndex();
}

/*********************************************************/
/* LOG View                                              */
YTTLinkLogView::YTTLinkLogView(QWidget *parent)
    : QWidget(parent), ui(new Ui::YTTLinkLogViewClass)
{
    ui->setupUi(this);
    icoConnect = new QIcon(":/images/connect.png");
    icoDisconnect = new QIcon(":/images/disconnect.png");
    icoError = new QIcon(":/images/error.png");
    icoWarning = new QIcon(":/images/warn.png");
    icoInfo = new QIcon(":/images/info.png");
    icoSend = new QIcon(":/images/up.png");
    icoReceive = new QIcon(":/images/down.png");

    connect(ui->ButtonSend, SIGNAL(clicked()), this, SLOT(slot_ButtonSendClicked()));


    if(NO_LOG){}
    else
    {
	logFile = new QFile("log.txt");
	if(!logFile->open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Append))
	    AddLine(iError, QString(tr("Could not write log file!")));
	else
	{
	    QTextStream logOutput(logFile);
	    logOutput<<QString(tr("\n\nSession from (d/m/y) ")).append(date.currentDate().toString(QString("dd/MM/yyyy"))).append("\n").toAscii();
	    while (ui->log->count() > 5)
		ui->log->takeItem(0);
	    ui->log->scrollToBottom();
	}
    }
}

YTTLinkLogView::~YTTLinkLogView()
{
    delete ui;
}

void YTTLinkLogView::slot_AddLine(int type, QString text)
{
    AddLine(type, text);
}

void YTTLinkLogView::AddLine(int type, QString text)
{
    QIcon *ico;
    QString txt;
    switch((ICOTYPE)type)
    {
	case iConnect:
	    ico = icoConnect;
	    break;
	case iDisconnect:
	    ico = icoDisconnect;
	    break;
	case iError:
	    ico = icoError;
	    break;
	case iWarning:
	    ico = icoWarning;
	    break;
	case  iSend:
	    ico = icoSend;
	    break;
	case iReceive:
	    ico = icoReceive;
	    break;
	case iInformation:
	default:
	    ico = icoInfo;
	    break;
    }
    //ui->listWidget->addItem(new QListWidgetItem(*ico, text, NULL, 0));
    txt = time.currentTime().toString("[hh:mm:ss]  ").append(text);
    if(txt.toAscii().endsWith("\n"))
	txt = txt.left(txt.length()-1);
    if(txt.toAscii().endsWith("\r"))
	txt = txt.left(txt.length()-1);
    ui->log->addItem(new QListWidgetItem(*ico, txt, NULL, 0));
    if(logFile->isOpen() & !NO_LOG)
    {
	QTextStream logOutput(logFile);
	logOutput<<date.currentDate().toString(QString("dd.MM.yyyy ")).append(txt).append("\n").toAscii();
	while (ui->log->count() > 50)
	    ui->log->takeItem(0);
	ui->log->scrollToBottom();
    }
}


void YTTLinkLogView::slot_WriteRawText(QString text)
{
    ui->Console->setText(ui->Console->toPlainText().append(text));
    ui->Console->moveCursor(QTextCursor::End);
    ui->Console->ensureCursorVisible();
    if(ui->Console->toPlainText().length() > 1000)
	ui->Console->setText(ui->Console->toPlainText().right(500));
}

void YTTLinkLogView::slot_ButtonSendClicked()
{
    QString output;
    if(ui->LineToSend->text().length()==0)
	return;
    AddLine(iInformation, QString(tr("Manual input: \"")).append(ui->LineToSend->text()).append("\""));
    output = ui->LineToSend->text();
    output.append("\r\n");
    emit sig_DataToSend(output);
//    emit sig_DataToSend(ui->LineToSend->text());
}
