/********************************************************************************************/
/*  class YTTLINK                                                                           */
/*                                                                                          */
/*  Encapsulates a link between a board and YTT                                             */
/*                                                                                          */
/*                                                                                          */
/*                                                                                          */
/*                                                                                          */
/********************************************************************************************/
#ifndef YTTLINK_H
#define YTTLINK_H

#include <QtGui/QWidget>
#include <QtGui/QTabWidget>
#include <QtGui/QMainWindow>
#include <QtGui/QWidget>
#include <QTime>
#include <QDate>
#include <QFile>
#include <QTextStream>
#include <QTimer>
#include "../ui_serconn.h"
#include "../ui_logview.h"
#include "../board/board.h"
#include "gui_10V/main10V.h"
#include <qextserialport.h>
#include <qextserialbase.h>

class BOARD;
class QextSerialPort;

enum LINKSTATUS {sInfo, sErr};
enum ICOTYPE {iConnect, iDisconnect, iWarning, iInformation, iError, iSend, iReceive};


/********************************************************************************************/
/*                                                                                          */
/*                                      For GUI                                             */
/*                                                                                          */
/********************************************************************************************/
namespace Ui
{
    class LinkSerialWidget;
}

class LinkSerialWidget : public QWidget
{
    Q_OBJECT

public:
    LinkSerialWidget(QWidget *parent = 0);
    ~LinkSerialWidget();

    int Type();

    void AddPortNum(QString portFriendlyName, QString portName);

//private:
    Ui::LinkSerialWidgetClass *ui;
};

/*********************************************************/
/* Log Console                                           */
namespace Ui
{
    class YTTLinkLogView;
}

class YTTLinkLogView : public QWidget
{
    Q_OBJECT

public:
    YTTLinkLogView(QWidget *parent = 0);
    ~YTTLinkLogView();
    void AddLine(int type, QString text);

public slots:
    void slot_AddLine(int type=-1, QString text=QString(""));
    void slot_WriteRawText(QString text);
    void slot_ButtonSendClicked();

signals:
    void sig_DataToSend(QString);
private:
    Ui::YTTLinkLogViewClass *ui;

    QTime time;
    QDate date;
    QFile *logFile;
    //QTextStream *logOutput;

    /* Log Viewer icon set */
    QIcon *icoConnect;
    QIcon *icoDisconnect;
    QIcon *icoError;
    QIcon *icoWarning;
    QIcon *icoInfo;
    QIcon *icoSend;
    QIcon *icoReceive;

    ICOTYPE type;


};



/********************************************************************************************/
/*                                                                                          */
/*                                     Link class                                           */
/*                                                                                          */
/********************************************************************************************/

class YTTLINK: public QObject{
    Q_OBJECT
	
public:

    enum STATUS {closed, opened};
    STATUS status;

    BOARD *board;
    YTTLinkLogView *logViewer;	// log viewer
    LinkSerialWidget *LinkMenu;
    ICOTYPE type;
    QDockWidget *logToolBox;


    YTTLINK(QObject *par);
    ~YTTLINK();

    int Close(void);							// return 0 on success
    bool IsOpen(void);							// return 1 if link opened
    void Send(QString text);
    QWidget* Menu();
    int BoardNumber();
    QWidget* ConfigWidget(int boardNum, int subPart);


public slots:
    int slot_Send(QString text);
    int slot_SendLine(QString text);
    void slot_LogConsole();                                 // makers the log and console window pop
    int Open();			// return 0 on success
    void verifyNewData();
    void connectionTimeout();
    void slot_boardModelChanged(int a);
signals:
    void sig_Receive(QString text);
    void sig_StatusChanged(int type, QString statusText);
    void sig_newLineReceived(QString newLine);

private:
    QextSerialPort *port;
    QTimer *timer;
    QTimer *timeoutTimer;
    QString receivedData;
};

/*QStringList errors({"E_NO_ERROR : No Error has occured",
		    "E_INVALID_FD : Invalid file descriptor (port was not opened correctly)",
		    "E_NO_MEMORY : Unable to allocate memory tables (POSIX)",
		    "E_CAUGHT_NON_BLOCKED_SIGNAL : ",
		    "E_PORT_TIMEOUT : ",
		    "E_INVALID_DEVICE : ",
		    "E_BREAK_CONDITION : ",
		    "E_FRAMING_ERROR : ",
		    "E_IO_ERROR :",
		    "E_BUFFER_OVERRUN :",
		    "E_RECEIVE_OVERFLOW :",
		    "E_RECEIVE_PARITY_ERROR :",
		    "E_TRANSMIT_OVERFLOW :",
		    "E_READ_FAILED :",
		    "E_WRITE_FAILED :",});*/


#endif // YTTLINK_H


